@@ -12,9 +12,9 @@ require 'shellwords'
 
 raise "sorry, 1.8 was last decade" unless RUBY_VERSION >= '1.9'
 
-gem 'kramdown', '~> 2.3.0'
+gem 'kramdown', '~> 2.4.0'
 require 'kramdown'
-my_span_elements =  %w{list figure xref eref iref cref spanx vspace}
+my_span_elements =  %w{list xref eref iref cref spanx vspace}
 Kramdown::Parser::Html::Constants::HTML_SPAN_ELEMENTS.concat my_span_elements
 
 require 'rexml/parsers/baseparser'
@@ -22,6 +22,8 @@ require 'open3'                 # for math
 require 'json'                  # for math
 require 'rexml/document'        # for SVG and bibxml acrobatics
 
+require 'kramdown-rfc/doi'      # for fetching information for a DOI
+
 class Object
   def deep_clone
     Marshal.load(Marshal.dump(self))
@@ -37,7 +39,7 @@ module Kramdown
       def replace_abbreviations(el, regexps = nil)
         unless regexps          # DUPLICATED AND MODIFIED CODE FROM UPSTREAM, CHECK ON UPSTREAM UPGRADE
           sorted_abbrevs = @root.options[:abbrev_defs].keys.sort {|a, b| b.length <=> a.length }
-          regexps = [Regexp.union(*sorted_abbrevs.map {|k| 
+          regexps = [Regexp.union(*sorted_abbrevs.map {|k|
                                     /#{Regexp.escape(k).gsub(/\\\s/, "[\\s\\p{Z}]+").force_encoding(Encoding::UTF_8)}/})]
           # warn regexps.inspect
           regexps << /(?=(?:\W|^)#{regexps.first}(?!\w))/ # regexp should only match on word boundaries
@@ -113,7 +115,7 @@ module Kramdown
         end
       end
 
-      XREF_START = /\{\{(?:(?:\{(.*?)\}(?:\{(.*?)\})?)|(\X*?))((?:\}\})|\})/u
+      XREF_START = /\{\{(?:(?:\{(.*?\n??.*?)\}(?:\{(.*?\n??.*?)\})?)|(\X*?))((?:\}\})|\})/u
 
       # Introduce new {{target}} syntax for empty xrefs, which would
       # otherwise be an ugly ![!](target) or ![ ](target)
@@ -124,9 +126,9 @@ module Kramdown
           warn "*** #{@src[0]}: unmatched braces #{@src[4].inspect}"
         end
         if contact_name = @src[1]
-          attr = {'fullname' => contact_name}
+          attr = {'fullname' => contact_name.gsub("\n", " ")}
           if ascii_name = @src[2]
-            attr["asciiFullname"] = ascii_name
+            attr["asciiFullname"] = ascii_name.gsub("\n", " ")
           end
           el = Element.new(:contact, nil, attr)
         else
@@ -242,6 +244,14 @@ module Kramdown
     TRUTHY["false"] = false
     TRUTHY["no"] = false
 
+    # explicit or automatic studlification
+    # note that explicit (including trailing "_") opts out of automatic
+    def self.attrmangle(k)
+      if (d = k.gsub(/_(.|$)/) { $1.upcase }) != k or d = STUDLY_ATTR_MAP[k]
+        d
+      end
+    end
+
     def rfc2629_fix(opts)
       if a = attr
         if anchor = a.delete('id')
@@ -254,7 +264,7 @@ module Kramdown
           opts = opts.merge(noabbrev: TRUTHY[av]) # updated copy
         end
         attr.keys.each do |k|
-          if (d = k.gsub(/_(.|$)/) { $1.upcase }) != k or d = STUDLY_ATTR_MAP[k]
+          if d = self.class.attrmangle(k)
             a[d] = a.delete(k)
           end
         end
@@ -713,8 +723,13 @@ COLORS
           if sl = el.attr.delete('slugifiedName') # could do general name- play
             attrstring = html_attributes({'slugifiedName' => sl})
           end
-          # noabbrev: true -- Workaround for https://trac.ietf.org/trac/xml2rfc/ticket/683
-          irefs = "<name#{attrstring}>#{inner(el, indent, opts.merge(noabbrev: true))}</name>" #
+          # noabbrev: true -- Workaround for https://github.com/ietf-tools/xml2rfc/issues/683
+          nm = inner(el, indent, opts.merge(noabbrev: true))
+          if ttl = el.attr['title']
+            warn "*** Section has two titles: >>#{ttl}<< and >>#{nm}<<"
+            warn "*** Do you maybe have a loose IAL?"
+          end
+          irefs = "<name#{attrstring}>#{nm}</name>" #
         else
         clean, irefs = clean_pcdata(inner_a(el, indent, opts))
         el.attr['title'] = clean
@@ -800,7 +815,7 @@ COLORS
             out ="#{' '*indent}<dd><t/></dd>\n" # you can't make this one up
           end
           opts[:haddt] = true
-          out << "#{' '*indent}<dt#{el_html_attributes(el)}>\n#{inner(el, indent, opts)}#{' '*indent}</dt>\n"
+          out << "#{' '*indent}<dt#{el_html_attributes(el)}>#{inner(el, indent, opts)}</dt>\n"
         else
         close = "#{' '*indent}</t>\n" * @in_dt
         @in_dt = 1
@@ -906,6 +921,17 @@ COLORS
         gi = el.attr.delete('gi')
         res = inner(el, indent, opts)
         target = el.attr['target']
+        if target[0..1] == "{{"
+          # XXX ignoring all attributes and content
+          s = ::Kramdown::Converter::Rfc2629::process_markdown(target)
+          # if res != '' && s[-2..-1] == '/>'
+          #   if s =~ /\A<([-A-Za-z0-9_.]+) /
+          #     gi ||= $1
+          #   end
+          #   s[-2..-1] = ">#{res}</#{gi}>"
+          # end
+          return s
+        end
         if target[0] == "#"     # handle [](#foo) as xref as in RFC 7328
           el.attr['target'] = target = target[1..-1]
           if target.downcase == res.downcase
@@ -978,6 +1004,12 @@ COLORS
         warn "(#{"%.3f" % (t2 - t1)} s)" if KRAMDOWN_PERSISTENT_VERBOSE
       end
 
+      def get_doi(refname)
+        lit = doi_fetch_and_convert(refname, fuzzy: true)
+        anchor = "DOI_#{refname.gsub("/", "_")}"
+        KramdownRFC::ref_to_xml(anchor, lit)
+      end
+
       # this is now slightly dangerous as multiple urls could map to the same cachefile
       def get_and_cache_resource(url, cachefile, tvalid = 7200, tn = Time.now)
         fn = "#{REFCACHEDIR}/#{cachefile}"
@@ -992,7 +1024,17 @@ COLORS
             fetch_timeout = 60 # seconds; long timeout needed for Travis
           end
           $stderr.puts "#{fn}: #{message} from #{url}"
-          if ENV["HAVE_WGET"]
+          if Array === url
+            begin
+              case url[0]
+              when :DOI
+                ref = get_doi(url[1])
+                File.write(fn, ref)
+              end
+            rescue Exception => e
+              warn "*** Error fetching #{url[0]} #{url[1].inspect}: #{e}"
+            end
+          elsif ENV["HAVE_WGET"]
             `cd #{REFCACHEDIR}; wget -t 3 -T #{fetch_timeout} -Nnv "#{url}"` # ignore errors if offline (hack)
             begin
               File.utime nil, nil, fn
@@ -1033,7 +1075,7 @@ COLORS
       def self.bcp_std_ref(t, n)
         warn "*** #{t} anchors not supported in v2 format" unless $options.v3
         [name = "reference.#{t}.#{"%04d" % n.to_i}.xml",
-         "#{XML_RESOURCE_ORG_PREFIX}/bibxml-rfcsubseries-new/#{name}"] # FOR NOW
+         "#{XML_RESOURCE_ORG_PREFIX}/bibxml-rfcsubseries/#{name}"] # FOR NOW
       end
 
       # [subdirectory name, cache ttl in seconds, does it provide for ?anchor=]
@@ -1065,12 +1107,14 @@ COLORS
         "NIST" => "bibxml2",
         "OASIS" => "bibxml2",
         "PKCS" => "bibxml2",
-        "DOI" => ["bibxml7", 86400, true], # 24 h cache at source anyway
+        "DOI" => ["bibxml7", 86400, true, ->(fn, n){ [fn, [:DOI, n] ] }
+                 ], # emulate old 24 h cache
         "IANA" => ["bibxml8", 86400, true], # ditto
       }
 
       # XML_RESOURCE_ORG_HOST = ENV["XML_RESOURCE_ORG_HOST"] || "xml.resource.org"
-      XML_RESOURCE_ORG_HOST = ENV["XML_RESOURCE_ORG_HOST"] || "xml2rfc.tools.ietf.org"
+      # XML_RESOURCE_ORG_HOST = ENV["XML_RESOURCE_ORG_HOST"] || "xml2rfc.tools.ietf.org"
+      XML_RESOURCE_ORG_HOST = ENV["XML_RESOURCE_ORG_HOST"] || "bib.ietf.org"
       XML_RESOURCE_ORG_PREFIX = ENV["XML_RESOURCE_ORG_PREFIX"] ||
                                 "https://#{XML_RESOURCE_ORG_HOST}/public/rfc"
       KRAMDOWN_USE_TOOLS_SERVER = ENV["KRAMDOWN_USE_TOOLS_SERVER"]
